'use strict';

/**
 * webpack css compression plugin
 * plugins: [require('webpack-css-min')]
 */

const regCss = /\.css$/;

module.exports = {
    apply: (compiler) => {
        compiler.plugin('emit', (compilation, callback) => {
            const assets = compilation.assets;
            Object
                .keys(assets)
                .filter((name) => regCss.test(name))
                .forEach((name) => {
                    let min = assets[name]
                        .source()
                        .replace(/\/\*[\d\D]*?\*\//g, '')
                        .replace(/\/\*\*\//g, '')
                        .replace(/\s+/g, ' ')
                        .replace(/ *([{};,>+~]) */g, '$1')
                        .replace(/ *: *([^{]*?[;}])/g, ':$1')
                        .replace(/ +!important/ig, '!important')
                        .replace(/@import +url/ig, '@import url')
                        .replace(/\b0px\b/g, '0')
                        .replace(/(\dpx) /g, '$1\n')
                        .replace(/;}/g, '}')
                        .replace(/@media.*?{/g, function (match) {
                            return match.replace(/(\W) /g, '$1').replace(/ (\W)/g, '$1');
                        })
                        .trim();

                    assets[name] = {
                        source: () => min,
                        size: () => min.length
                    };
                });

            callback();
        });
    }
};